from ..services import client,printer,util,oss_client
from terminal import white, yellow
from ..const import COMMAND
import time
from blessings import Terminal

def getDetail(job_id, task_name=None, instance_id=None, limit=20, watch=False):

    if task_name==None:
        show_job_detail(job_id, watch)

    elif instance_id==None:
        show_task_detail(job_id, task_name, watch)

    else:
        show_instance_detail(job_id, task_name, instance_id, limit)

def show_job_detail(job_id, watch):

    if watch:

        try:
            printer.print_watching_tip(3)
            opt = get_job_detail(job_id)
            n = print_job_detail(job_id, opt)

            time.sleep(3)
            term = Terminal()
            while True:
                with term.location(0, term.height - n - 2):
                    opt = get_job_detail(job_id)
                    print(term.clear_eos + term.move_up)

                    printer.print_watching_tip(3)
                    n = print_job_detail(job_id, opt)

                    state = opt['job_list'][0]['State']
                    if state in ['Finished','Failed']:
                        print(yellow('\nJob state is %s, quit watching mode!\n' % state))
                        break

                    time.sleep(3)
        except KeyboardInterrupt as e:
            print('KeyboardInterrupt')

    else:
        opt = get_job_detail(job_id)
        print_job_detail(job_id, opt)
        printer.tip('Type "%s job %s <TaskName>" to show task detail' % (COMMAND, job_id))


def get_job_detail(job_id):

    job = client.get_job(job_id)

    job_list = [job]
    # format Date
    job_list = printer.format_date_fields(job_list,['StartTime','EndTime'], util.JOB_KEYS)
    # calc Progress
    job_list = util.calc_job_progress(job_list)



    tasks = client.list_tasks(job_id)
    # format Date
    task_list = printer.format_date_fields(tasks,['StartTime', 'EndTime'], util.TASK_KEYS)
    # calc Progress
    task_list = util.calc_task_progress(task_list)

    desc = client.get_job_description(job_id)

    return {'job_list':job_list,'job_desc':desc, 'task_list':task_list}

def print_job_detail(job_id, opt):
    n = 0
    n += printer.print_title('Job: %s' % job_id)
    n += printer.print_arr(opt['job_list'], util.JOB_COLS[1:]  + ['Priority'], False)

    n += printer.print_deps(opt['job_desc'].get('TaskDag').get('Dependencies'))

    n += printer.print_title('Tasks:')

    n += printer.print_arr(opt['task_list'], util.TASK_COLS, False)
    return n



def show_task_detail(job_id, task_name, watch):
    if watch:
        try:
            printer.print_watching_tip(3)
            opt = get_task_detail(job_id, task_name)
            n = print_task_detail(task_name, opt)

            time.sleep(3)
            term = Terminal()
            while True:
                with term.location(0, term.height - n - 2):
                    opt = get_task_detail(job_id, task_name)
                    print(term.clear_eos + term.move_up)

                    printer.print_watching_tip(3)
                    n = print_task_detail(task_name, opt)

                    state = opt['task_list'][0]['State']
                    if state in ['Finished','Failed']:
                        print(yellow('\nTask state is %s, quit watching mode!\n' % state))
                        break

                    time.sleep(3)
        except KeyboardInterrupt as e:
            print('KeyboardInterrupt')


    else:
        opt = get_task_detail(job_id, task_name)
        print_task_detail(task_name, opt)
        printer.tip('Type "%s job %s %s <InstanceId>" to get instance log list' % (COMMAND, job_id, task_name))

def get_task_detail(job_id, task_name):

    #################
    des = client.get_job_description(job_id)
    task_desc = des.get('TaskDag').get('TaskDescMap').get(task_name)

    if not task_desc:
        raise Exception('Not found task: %s' % task_name)

    #printer.print_arr([util.to_dict(task_desc,util.TASK_DESC_COLS)], util.TASK_DESC_COLS)

    ################
    tasks = client.list_tasks(job_id)
    task = [item for item in tasks if item.TaskName==task_name][0]


    # format Date
    task_list = printer.format_date_fields([task],['StartTime', 'EndTime'], util.TASK_KEYS)
    # calc Progress
    task_list = util.calc_task_progress(task_list)


    stateCount = task_list[0].get('StateCount')
    stateStr = util.state_count_flat(stateCount)

    ################
    inst_list = task['InstanceList']
    return {'task_desc':task_desc, 'task_list':task_list, 'stateStr':stateStr, 'inst_list':inst_list}

def print_task_detail(task_name, opt):
    n = 0
    n += printer.print_title('Task Description:')
    #print task_desc
    n += printer.print_task_desc(task_name, opt['task_desc'])

    n += printer.print_title('Task Status:')
    # print task
    n += printer.print_arr(opt['task_list'], util.TASK_COLS, False)

    # instance list
    n += printer.print_title('Instances: %s' % opt['stateStr'])
    print('%s' % white('-'*90))
    n += 1
    n += printer.print_instance_blocks(opt['inst_list'])

    return n


def show_instance_detail(job_id, task_name, instance_id, limit):
    des = client.get_job_description(job_id)
    task_desc = des.get('TaskDag').get('TaskDescMap').get(task_name)

    if not task_desc:
        raise Exception('Not found task: %s' % task_name)

    stdout_pre = '%s%s/stdout.%s' % (oss_dir_path(task_desc.get('StdoutRedirectPath')), task_name, instance_id)
    stderr_pre = '%s%s/stderr.%s' % (oss_dir_path(task_desc.get('StderrRedirectPath')), task_name, instance_id)

    arr1 = oss_client.list(stdout_pre)
    arr2 = oss_client.list(stderr_pre)

    for k in arr2:
        arr1.append(k)
    arr1.sort(key=lambda x:x['time'],reverse=True)

    printer.print_title('Instance Log (last %d rows):' % limit)
    printer.print_arr(arr1[:limit], ['log_path','time','size'])
    printer.tip('Type "%s log <log_path>" to get log content.' % COMMAND)

def oss_dir_path(s):
    if(s.endswith('/')):
        return s
    else:
        return '%s/' % s


def transform_limit(limit=20):
    try:
        return int(limit)
    except:
        raise Exception('--limit must be a Positive interger')
