import re

from . import client, oss_client

def validate_job(job):
    t = []

    job_name = job.get('JobName')
    if not check_JobName(job_name):
        t.append('invalid JobName: %s' % job_name)

    priority = job.get('Priority')
    if not isinstance(priority, int) or priority > 1000 or priority < 0:
        t.append('invalid Priority, it should be an integer in [0,1000]')

    #tasks
    if not job.get('TaskDag') or not job['TaskDag'].get('TaskDescMap'):
        t.append('Not found TaskDescMap property')

    tasks = job['TaskDag'].get('TaskDescMap')

    IMG_ARR = [n['ImageId'] for n in client.list_images()]

    for k in tasks:
        if not check_TaskName(k):
            t.append('Invalid TaskName: %s' % k)

        if not check_ImageId(tasks[k].get('ImageId'), IMG_ARR):
            t.append('Invalid %s.ImageId: %s' % (k,tasks[k].get('ImageId')))

        if not check_InstanceCount(tasks[k].get('InstanceCount')):
            t.append('Invalid %s.InstanceCount: %s' % (k,tasks[k].get('InstanceCount')))

        if not check_ProgramType(tasks[k].get('ProgramType')):
            t.append('Invalid %s.ProgramType: %s' % (k, tasks[k].get('ProgramType')))


        if not check_oss_path_format(tasks[k].get('PackageUri')):
            t.append('Invalid %s.PackageUri: %s'% (k,tasks[k].get('PackageUri')) )

        if not check_oss_path_format(tasks[k].get('StdoutRedirectPath')):
            t.append('Invalid %s.StdoutRedirectPath: %s'% (k,tasks[k].get('StdoutRedirectPath')) )

        if not check_oss_path_format(tasks[k].get('StderrRedirectPath')):
            t.append('Invalid %s.StderrRedirectPath: %s'% (k,tasks[k].get('StderrRedirectPath')) )

        if not tasks[k].get('ResourceDescription'):
            t.append('Invalid %s.ResourceDescription'% k )
        else:
            if not check_cpu( tasks[k]['ResourceDescription'].get('Cpu')):
                t.append('Invalid %s.CPU: %s'% (k,tasks[k]['ResourceDescription'].get('Cpu')) )
            if not check_memory(tasks[k]['ResourceDescription'].get('Memory')):
                t.append('Invalid %s.Memory: %s' % (k,tasks[k]['ResourceDescription'].get('Memory')) )


        if tasks[k].get('OssMapping'):
            mappings = tasks[k].get('OssMapping')
            for k2 in mappings:
                if not check_oss_path(k2):
                    t.append('Invalid %s.OssMapping: %s'% (k, k2) )


    if len(t)==0:
        return False
    else:
        return '\n'.join(t)


def check_JobName(job_name):
    return re.match(r'[\w\-]{1,63}$', job_name)!=None


def check_TaskName(task_name):
    return re.match(r'[\w\-]{1,63}$', task_name)!=None

def check_ImageId(image_id, IMG_ARR):
    return image_id in IMG_ARR

def check_InstanceCount(c):
    return isinstance(c, int) and c > 0

def check_ProgramType(type):
    return type in ['native','python','java']

def check_oss_path_format(oss):
    return oss and oss.startswith('oss://') and oss.find('/', 7)

def check_cpu(cpu):
    return isinstance(cpu, int) and cpu >= 800

def check_memory(mem):
    return isinstance(mem, int) and mem >= 1600

def check_oss_path(p):
    if not p:
        return False

    try:
        oss_client.check(p)
        return True
    except:
        return False





